import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;

import junit.framework.TestCase;

import org.hornetq.api.core.SimpleString;
import org.hornetq.core.filter.Filter;
import org.hornetq.core.filter.impl.FilterImpl;
import org.hornetq.core.server.ServerMessage;
import org.hornetq.core.server.impl.ServerMessageImpl;

public class TestFilter extends TestCase
{
   private static final SimpleString ADDRESS = SimpleString.toSimpleString("jms.topic.testTopic");

   private static final String SELECTOR_FIELD = "RECEIVER";

   private static final String CORRECT_VALUE_FOR_SELECTOR = "A";

   private static final String INCORRECT_VALUE_FOR_SELECTOR = "B";

   private static final String FILTER_STRING = SELECTOR_FIELD + "=" + "'" + CORRECT_VALUE_FOR_SELECTOR + "'";

   public void testCorrectMessageMatch() throws Exception
   {
      Filter filter = FilterImpl.createFilter(FILTER_STRING);

      ServerMessage goodMsg = new ServerMessageImpl();
      goodMsg.setAddress(ADDRESS);
      goodMsg.putStringProperty(SELECTOR_FIELD, CORRECT_VALUE_FOR_SELECTOR);
      goodMsg.putStringProperty("SENDER", "C"); // just additional property - no impact on the filter

      assertTrue("Correct message should match", filter.match(goodMsg));
   }

   public void testIncorrectMessageNotMatch() throws Exception
   {
      Filter filter = FilterImpl.createFilter(FILTER_STRING);

      ServerMessage badMsg = new ServerMessageImpl();
      badMsg.setAddress(ADDRESS);
      badMsg.putStringProperty(SELECTOR_FIELD, INCORRECT_VALUE_FOR_SELECTOR);
      badMsg.putStringProperty("SENDER", "C"); // just additional property - no impact on the filter

      assertFalse("Incorrect message should not match", filter.match(badMsg));
   }

   public void testFilterMultiThreaded() throws Exception
   {
      Filter filter = FilterImpl.createFilter(FILTER_STRING);

      ServerMessage goodMsg = new ServerMessageImpl();
      goodMsg.setAddress(ADDRESS);
      goodMsg.putStringProperty(SELECTOR_FIELD, CORRECT_VALUE_FOR_SELECTOR);
      goodMsg.putStringProperty("SENDER", "C");

      ServerMessage badMsg = new ServerMessageImpl();
      badMsg.setAddress(ADDRESS);
      badMsg.putStringProperty(SELECTOR_FIELD, INCORRECT_VALUE_FOR_SELECTOR);
      badMsg.putStringProperty("SENDER", "C"); // just additional property - no impact on the filter
      
      
      List<Callable<RuntimeException>> tasks = new ArrayList<Callable<RuntimeException>>();
      
      for(int i = 0; i < 10; i++)
      {
         tasks.add(new FilterCheckerTask(filter, badMsg, goodMsg));
      }
      
      ExecutorService executorService = Executors.newFixedThreadPool(10);
      
      List<Future<RuntimeException>> futures = executorService.invokeAll(tasks);
      List<RuntimeException> errors = new ArrayList<RuntimeException>();      
      for (Future<RuntimeException> future : futures) {
         RuntimeException exc = future.get();
         if(exc != null)
         {
            errors.add(exc);
            System.out.println(exc);
         }
      }
      assertTrue("Filter should accept the correct messages and discard incorrect ones", errors.isEmpty());
   }

   public static class FilterCheckerTask implements Callable<RuntimeException>
   {
      private Filter filter;

      private ServerMessage badMsg;

      private ServerMessage goodMsg;
      
      public FilterCheckerTask(Filter filter, ServerMessage badMsg, ServerMessage goodMsg)
      {
         this.filter = filter;
         this.badMsg = badMsg;
         this.goodMsg = goodMsg;
      }
      
      public RuntimeException call() throws Exception
      {
         for (int i = 0; i < 100000; i++)
         {
            if (filter.match(badMsg))
            {
               return new RuntimeException("bad message matches");
            }
            if (!filter.match(goodMsg))
            {
               return new RuntimeException("good message does not match");
            }
         }
         return null;
      }

   }
   
}
